package net.lecnam.info.remotelist;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import net.lecnam.info.channel.ObjectReader;
import net.lecnam.info.channel.ObjectWriter;
import net.lecnam.info.remotelist.Answer.*;
import net.lecnam.info.remotelist.Request.*;

public class Server<E> implements Runnable {
	
	ObjectReader<Request<E>> in;
	ObjectWriter<Answer<E>> out;
	List<E> l;
	
	static final Logger logger = Logger.getLogger("server");
	
	Server(ObjectReader<Request<E>> in, ObjectWriter<Answer<E>> out, List<E> l) {
		this.in = in;
		this.out = out;
		this.l = l;
	}
	
	@Override
	public void run() {
		try {
			boolean done = false;
			while (!done) {
				Request<E> request = in.readObject();
				logger.info("Request: " + request);
				switch (request) {
					case Stop() -> {
						done = true;
					}
					case GetSize() -> {
						int s = l.size();
						Answer<E> answer = new Size<>(s);
						out.writeObject(answer);
					}
					case GetValue(var index) -> {
						E elt = l.get(index);
						Answer<E> answer = new Value<>(elt);
						out.writeObject(answer);
					}
					case Set(var index, var element) -> {
						E old = l.set(index, element);
						Answer<E> answer = new OldValue<>(old);
						out.writeObject(answer);
					}						
					case Add(var index, var element) -> {
						l.add(index, element);
						Answer<E> answer = new Done<>();
						out.writeObject(answer);						
					}
					case Remove(var index) -> {
						E old = l.remove(index);
						Answer<E> answer = new OldValue<>(old);
						out.writeObject(answer);					
					}
				}
			}
		} catch (IOException ex) {
			throw new Error(ex);
		}
	}
	
}
